
#if HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <atm.h>


#define SEND 100
#define RECV 140
#define RECOVER 10


static const char *rotor[] = { "|\r","/\r","-\r","\\\r" };

struct timeval sendd = {0, SEND};
struct timeval recvd = {0, RECV}; 

static void usage(const char *name)
{
    fprintf(stderr,"usage: %s [itf.]vpi.vci\n",name);
    exit(1);
}


int isbefore(struct timeval *t1, struct timeval *t2)
{
    if  ((t1->tv_sec < t2->tv_sec) || 
        ((t1->tv_sec == t2->tv_sec) &&  (t1->tv_usec < t2->tv_usec)))
        return 1;
    else
        return 0;
}

void timeadd(struct timeval *td, struct timeval *t1, struct timeval *t2)
{

    td->tv_sec = t1->tv_sec + t2->tv_sec;
    td->tv_usec = t1->tv_usec + t2->tv_usec;
    if (td->tv_usec >= 1000000) {
        td->tv_sec++;
        td->tv_usec -= 1000000;
    }

}

void timesub(struct timeval *td, struct timeval *t1, struct timeval *t2)
{
    td->tv_sec = t1->tv_sec - t2->tv_sec;
    td->tv_usec = t1->tv_usec - t2->tv_usec;

    if (td->tv_usec < 0) {
        td->tv_sec--;
        td->tv_usec += 1000000;
    }
}


int main(int argc,char **argv)
{
    struct timeval now, next, to, tmes, tacc, delta;
    struct sockaddr_atmpvc addr;
    struct atm_qos qos;
    int s,i, nrot = 0;
    unsigned long buf[13], pcr, us;
    int type = 1, gfc = 0, clp = 0; 

    delta.tv_sec = delta.tv_usec = 0;

    if (argc != 2) usage(argv[0]);
    if ((s = socket(PF_ATMPVC,SOCK_DGRAM,0)) < 0) {
        perror("socket");
        return 1;
    }
    memset(&addr,0,sizeof(addr));
    if (text2atm(argv[1],(struct sockaddr *) &addr,sizeof(addr),
      T2A_PVC | T2A_UNSPEC | T2A_WILDCARD) < 0) 
        usage(argv[0]);

    memset(&qos,0,sizeof(qos));
    qos.aal = ATM_AAL5;
    qos.txtp.traffic_class = ATM_UBR;
    qos.txtp.max_sdu = 52;
    qos.rxtp = qos.txtp;

    if (setsockopt(s, SOL_ATM, SO_ATMQOS, &qos,sizeof(qos)) < 0) {
        perror("setsockopt SO_ATMQOS");
        return 1;
    }
    if (bind(s,(struct sockaddr *) &addr,sizeof(addr)) < 0) {
        perror("bind");
        return 1;
    }
    if (gettimeofday(&next,NULL) < 0) {
        perror("gettimeofday");
        return 1;
    }
    to = next;
    tmes = next;
    to.tv_sec += RECOVER;
    i = 0;
    while (1) {
        fd_set set;

        FD_ZERO(&set);
        FD_SET(s,&set);
        select(s+1,&set,NULL,NULL,&delta);

        if (gettimeofday(&now,NULL) < 0) {
            perror("gettimeofday");
            return 1;
        }
        if (FD_ISSET(s, &set)) {
            int size;

            size = read(s, buf, 52);
            if (size < 0) {
                perror("read");
                return 1;
            }
            if (size != 52) {
                fprintf(stderr,"bad RX (%d)\n",size);
            }
            timeadd (&to, &to, &recvd);
            if ((++nrot % 1000) == 0) {
                tmes.tv_sec = now.tv_sec - tmes.tv_sec;
                tmes.tv_usec = now.tv_usec - tmes.tv_usec;
                if (tmes.tv_usec >= 1000000) {
                    timesub (&tacc, &now, &tmes); 
                    us = tacc.tv_sec * 1000000 + tacc.tv_usec;
                    pcr = 1000000000ul / us;
                    fprintf(stderr, "PCR = %lu\n", pcr);
                    tmes = now; 
                }
            }
                    
        }
        if (isbefore (&to, &now)) {
            fprintf(stderr,"RX timed out\n");
            to.tv_sec += RECOVER;
        }

        while (isbefore(&next, &now)) {
            buf[0] = (gfc << ATM_HDR_GFC_SHIFT) | (addr.sap_addr.vpi << ATM_HDR_VPI_SHIFT) |
            (addr.sap_addr.vci << ATM_HDR_VCI_SHIFT) | (type << ATM_HDR_PTI_SHIFT) | clp;
            
            if (write(s, buf, ATM_CELL_SIZE-1) != ATM_CELL_SIZE-1) {
                perror("write");
                return 1;
            }
            timeadd (&next, &next, &sendd);
        }
    }
    return 0;
}
